@@ -1,19 +1,26 @@
 /*
-   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
-
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
 
 #ifndef COPY_FRAG_HPP
 #define COPY_FRAG_HPP
@@ -146,7 +153,8 @@ struct PrepareCopyFragRef
 
 struct PrepareCopyFragConf
 {
-  STATIC_CONST( SignalLength = 7 );
+  STATIC_CONST( OldSignalLength = 7 );
+  STATIC_CONST( SignalLength = 8 );
 
   Uint32 senderRef;
   Uint32 senderData;
@@ -155,8 +163,78 @@ struct PrepareCopyFragConf
   Uint32 copyNodeId;
   Uint32 startingNodeId;
   Uint32 maxPageNo;
+  Uint32 completedGci;
 };
 
+class HaltCopyFragReq
+{
+  friend class Dblqh;
+  STATIC_CONST( SignalLength = 4);
+
+  Uint32 senderRef;
+  Uint32 senderData;
+  Uint32 tableId;
+  Uint32 fragmentId;
+};
+
+class HaltCopyFragConf
+{
+  friend class Dblqh;
+  STATIC_CONST( SignalLength = 4);
+
+  enum
+  {
+    COPY_FRAG_HALTED = 0,
+    COPY_FRAG_COMPLETED = 1
+  };
+  Uint32 senderData;
+  Uint32 tableId;
+  Uint32 fragmentId;
+  Uint32 cause;
+};
+
+class HaltCopyFragRef
+{
+  friend class Dblqh;
+  STATIC_CONST( SignalLength = 4);
+
+  Uint32 senderData;
+  Uint32 tableId;
+  Uint32 fragmentId;
+  Uint32 errorCode;
+};
+
+class ResumeCopyFragReq
+{
+  friend class Dblqh;
+  STATIC_CONST( SignalLength = 4);
+
+  Uint32 senderRef;
+  Uint32 senderData;
+  Uint32 tableId;
+  Uint32 fragmentId;
+};
+
+class ResumeCopyFragConf
+{
+  friend class Dblqh;
+  STATIC_CONST( SignalLength = 3);
+
+  Uint32 senderData;
+  Uint32 tableId;
+  Uint32 fragmentId;
+};
+
+class ResumeCopyFragRef
+{
+  friend class Dblqh;
+  STATIC_CONST( SignalLength = 4);
+
+  Uint32 senderData;
+  Uint32 tableId;
+  Uint32 fragmentId;
+  Uint32 errorCode;
+};
 
 #undef JAM_FILE_ID
 
