@@ -1,15 +1,21 @@
 /*
-   Copyright (c) 2003-2006, 2008 MySQL AB, 2008 Sun Microsystems, Inc.
-   Use is subject to license terms.
+   Copyright (c) 2003, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -45,13 +51,22 @@ public:
 		  bool checksum, 
 		  bool signalId,
 		  key_t shmKey,
-		  Uint32 shmSize);
+		  Uint32 shmSize,
+		  bool preSendChecksum,
+                  Uint32 spintime,
+                  Uint32 send_buffer_size);
   
   /**
    * SHM destructor
    */
   virtual ~SHM_Transporter();
-  
+
+  /**
+   * Clear any data buffered in the transporter.
+   * Should only be called in a disconnected state.
+   */
+  virtual void resetBuffers();
+
   virtual bool configure_derived(const TransporterConfiguration* conf);
 
   /**
@@ -59,13 +74,14 @@ public:
    */
   bool initTransporter();
   
-  void getReceivePtr(Uint32 ** ptr, Uint32 ** eod){
-    reader->getReadPtr(* ptr, * eod);
+  void getReceivePtr(Uint32 ** ptr,
+                     Uint32 ** eod,
+                     Uint32 ** end)
+  {
+    reader->getReadPtr(* ptr, * eod, * end);
   }
   
-  void updateReceivePtr(Uint32 * ptr){
-    reader->updateReadPtr(ptr);
-  }
+  void updateReceivePtr(TransporterReceiveHandle&, Uint32 * ptr);
   
 protected:
   /**
@@ -75,6 +91,11 @@ protected:
    */
   void disconnectImpl();
 
+  /**
+   * Disconnect socket that was used for wakeup services.
+   */
+  void disconnect_socket();
+
   /**
    * Blocking
    *
@@ -108,6 +129,8 @@ protected:
   bool ndb_shm_create();
   bool ndb_shm_get();
   bool ndb_shm_attach();
+  void ndb_shm_destroy();
+  void set_socket(NDB_SOCKET_TYPE);
 
   /**
    * Check if there are two processes attached to the segment (a connection)
@@ -119,15 +142,41 @@ protected:
   /**
    * Initialises the SHM_Reader and SHM_Writer on the segment 
    */
-  void setupBuffers();
+  bool setupBuffers();
 
   /**
    * doSend (i.e signal receiver)
    */
-  int doSend();
+  bool doSend(bool need_wakeup = true);
+  void doReceive();
+  void wakeup();
+
+  /**
+   * When sending as client I use the server mutex and server status
+   * flag to check for server awakeness, so sender always uses the
+   * reverse state.
+   *
+   * When receiving I change my own state, so client updates client
+   * status flag and locks client mutex.
+   */
+  void lock_mutex();
+  void unlock_mutex();
+  void lock_reverse_mutex();
+  void unlock_reverse_mutex();
+  void set_awake_state(Uint32 awake_state);
+  bool handle_reverse_awake_state();
+
+  void remove_mutexes();
+  void setupBuffersUndone();
+
   int m_remote_pid;
   Uint32 m_signal_threshold;
 
+  Uint32 m_spintime;
+  Uint32 get_spintime()
+  {
+    return m_spintime;
+  }
 private:
   bool _shmSegCreated;
   bool _attached;
@@ -135,7 +184,20 @@ private:
   
   key_t shmKey;
   volatile Uint32 * serverStatusFlag;
-  volatile Uint32 * clientStatusFlag;  
+  volatile Uint32 * clientStatusFlag;
+
+  bool m_server_locked;
+  bool m_client_locked;
+
+  Uint32 *serverAwakenedFlag;
+  Uint32 *clientAwakenedFlag;
+
+  Uint32 *serverUpFlag;
+  Uint32 *clientUpFlag;
+
+  NdbMutex *serverMutex;
+  NdbMutex *clientMutex;
+
   bool setupBuffersDone;
   
 #ifdef NDB_WIN32
@@ -146,10 +208,13 @@ private:
   
   int shmSize;
   char * shmBuf;
-  
-  SHM_Reader * reader;
-  SHM_Writer * writer;
-  
+
+  SHM_Reader *reader;
+  SHM_Writer *writer;
+
+  SHM_Reader m_shm_reader;
+  SHM_Writer m_shm_writer;
+
   /**
    * @return - True if the reader has data to read on its segment.
    */
@@ -163,7 +228,8 @@ private:
   {
     return ((Uint32)bufsize >= m_signal_threshold);
   }
-  bool send_is_possible(int timeout_millisec) const { return 1; }
+  bool send_is_possible(int timeout_millisec) const;
+  void detach_shm(bool rep_error);
 };
 
 #endif
