@@ -32,7 +32,7 @@
 
 #include <Properties.hpp>
 #include <Configuration.hpp>
-
+#include <signaldata/RedoStateRep.hpp>
 #include <EventLogger.hpp>
 extern EventLogger * g_eventLogger;
 
@@ -94,6 +94,12 @@ Backup::Backup(Block_context& ctx, Uint32 instanceNumber) :
   addRecSignal(GSN_FSREMOVEREF, &Backup::execFSREMOVEREF, true);
   addRecSignal(GSN_FSREMOVECONF, &Backup::execFSREMOVECONF);
 
+  addRecSignal(GSN_FSREADREF, &Backup::execFSREADREF, true);
+  addRecSignal(GSN_FSREADCONF, &Backup::execFSREADCONF);
+
+  addRecSignal(GSN_FSWRITEREF, &Backup::execFSWRITEREF, true);
+  addRecSignal(GSN_FSWRITECONF, &Backup::execFSWRITECONF);
+
   /*****/
   addRecSignal(GSN_BACKUP_REQ, &Backup::execBACKUP_REQ);
   addRecSignal(GSN_ABORT_BACKUP_ORD, &Backup::execABORT_BACKUP_ORD);
@@ -123,11 +129,18 @@ Backup::Backup(Block_context& ctx, Uint32 instanceNumber) :
   addRecSignal(GSN_UTIL_SEQUENCE_REF, &Backup::execUTIL_SEQUENCE_REF);
   addRecSignal(GSN_UTIL_SEQUENCE_CONF, &Backup::execUTIL_SEQUENCE_CONF);
 
+  addRecSignal(GSN_REDO_STATE_REP, &Backup::execREDO_STATE_REP);
+
   addRecSignal(GSN_WAIT_GCP_REF, &Backup::execWAIT_GCP_REF);
   addRecSignal(GSN_WAIT_GCP_CONF, &Backup::execWAIT_GCP_CONF);
   addRecSignal(GSN_BACKUP_LOCK_TAB_CONF, &Backup::execBACKUP_LOCK_TAB_CONF);
   addRecSignal(GSN_BACKUP_LOCK_TAB_REF, &Backup::execBACKUP_LOCK_TAB_REF);
 
+  addRecSignal(GSN_RESTORABLE_GCI_REP, &Backup::execRESTORABLE_GCI_REP);
+  addRecSignal(GSN_INFORM_BACKUP_DROP_TAB_REQ,
+               &Backup::execINFORM_BACKUP_DROP_TAB_REQ);
+  addRecSignal(GSN_WAIT_LCP_IDLE_REQ, &Backup::execWAIT_LCP_IDLE_REQ);
+
   addRecSignal(GSN_LCP_STATUS_REQ, &Backup::execLCP_STATUS_REQ);
 
   /**
@@ -141,10 +154,55 @@ Backup::Backup(Block_context& ctx, Uint32 instanceNumber) :
   addRecSignal(GSN_LCP_PREPARE_REQ, &Backup::execLCP_PREPARE_REQ);
   addRecSignal(GSN_END_LCPREQ, &Backup::execEND_LCPREQ);
 
+  addRecSignal(GSN_SYNC_PAGE_WAIT_REP, &Backup::execSYNC_PAGE_WAIT_REP);
+  addRecSignal(GSN_SYNC_PAGE_CACHE_CONF, &Backup::execSYNC_PAGE_CACHE_CONF);
+  addRecSignal(GSN_SYNC_EXTENT_PAGES_CONF,
+               &Backup::execSYNC_EXTENT_PAGES_CONF);
+
   addRecSignal(GSN_DBINFO_SCANREQ, &Backup::execDBINFO_SCANREQ);
 
   addRecSignal(GSN_CHECK_NODE_RESTARTCONF,
                &Backup::execCHECK_NODE_RESTARTCONF);
+  {
+    CallbackEntry& ce = m_callbackEntry[THE_NULL_CALLBACK];
+    ce.m_function = TheNULLCallback.m_callbackFunction;
+    ce.m_flags = 0;
+  }
+  { // 1
+    CallbackEntry& ce = m_callbackEntry[SYNC_LOG_LCP_LSN];
+    ce.m_function = safe_cast(&Backup::sync_log_lcp_lsn_callback);
+    ce.m_flags = 0;
+  }
+  {
+    CallbackTable& ct = m_callbackTable;
+    ct.m_count = COUNT_CALLBACKS;
+    ct.m_entry = m_callbackEntry;
+    m_callbackTableAddr = &ct;
+  }
+  m_redo_alert_state = RedoStateRep::NO_REDO_ALERT;
+  m_local_redo_alert_state = RedoStateRep::NO_REDO_ALERT;
+  m_global_redo_alert_state = RedoStateRep::NO_REDO_ALERT;
+  m_max_redo_speed_per_sec = Uint64(0);
+  NdbTick_Invalidate(&m_lcp_start_time);
+  NdbTick_Invalidate(&m_prev_lcp_start_time);
+  NdbTick_Invalidate(&m_lcp_current_cut_point);
+  m_last_redo_used_in_bytes = Uint64(0);
+  m_last_lcp_exec_time_in_ms = Uint64(0);
+  m_update_size_lcp[0] = Uint64(0);
+  m_update_size_lcp[1] = Uint64(0);
+  m_update_size_lcp_last = Uint64(0);
+  m_insert_size_lcp[0] = Uint64(0);
+  m_insert_size_lcp[1] = Uint64(0);
+  m_insert_size_lcp_last = Uint64(0);
+  m_delete_size_lcp[0] = Uint64(0);
+  m_delete_size_lcp[1] = Uint64(0);
+  m_delete_size_lcp_last = Uint64(0);
+  m_proposed_disk_write_speed = Uint64(0);
+  m_lcp_lag[0] = Int64(0);
+  m_lcp_lag[1] = Int64(0);
+  m_lcp_timing_counter = Uint64(0);
+  m_lcp_change_rate = Uint64(0);
+  m_lcp_timing_factor = Uint64(100);
 }
   
 Backup::~Backup()
@@ -153,9 +211,6 @@ Backup::~Backup()
 
 BLOCK_FUNCTIONS(Backup)
 
-template class ArrayPool<Backup::Page32>;
-template class ArrayPool<Backup::Fragment>;
-
 void
 Backup::execREAD_CONFIG_REQ(Signal* signal)
 {
@@ -177,7 +232,8 @@ Backup::execREAD_CONFIG_REQ(Signal* signal)
   c_defaults.m_o_direct = true;
   c_defaults.m_backup_disk_write_pct = 50;
 
-  Uint32 noBackups = 0, noTables = 0, noAttribs = 0, noFrags = 0;
+  Uint32 noBackups = 0, noTables = 0, noFrags = 0;
+  Uint32 noDeleteLcpFile = 0;
   ndbrequire(!ndb_mgm_get_int_parameter(p, CFG_DB_DISCLESS, 
 					&c_defaults.m_diskless));
   ndb_mgm_get_int_parameter(p, CFG_DB_O_DIRECT,
@@ -203,6 +259,22 @@ Backup::execREAD_CONFIG_REQ(Signal* signal)
   ndb_mgm_get_int_parameter(p, CFG_DB_COMPRESSED_LCP,
 			    &c_defaults.m_compressed_lcp);
 
+  m_enable_partial_lcp = 1; /* Default to enabled */
+  ndb_mgm_get_int_parameter(p, CFG_DB_ENABLE_PARTIAL_LCP,
+                            &m_enable_partial_lcp);
+
+  m_enable_redo_control = 0; /* Default to disabled */
+  ndb_mgm_get_int_parameter(p, CFG_DB_ENABLE_REDO_CONTROL,
+                            &m_enable_redo_control);
+
+  m_recovery_work = 60; /* Default to 60% */
+  ndb_mgm_get_int_parameter(p, CFG_DB_RECOVERY_WORK, &m_recovery_work);
+
+  m_insert_recovery_work = 40; /* Default to 40% */
+  ndb_mgm_get_int_parameter(p,
+                            CFG_DB_INSERT_RECOVERY_WORK,
+                            &m_insert_recovery_work);
+
   calculate_real_disk_write_speed_parameters();
 
   jam();
@@ -213,44 +285,86 @@ Backup::execREAD_CONFIG_REQ(Signal* signal)
   ndb_mgm_get_int_parameter(p, CFG_DB_PARALLEL_BACKUPS, &noBackups);
   //  ndbrequire(!ndb_mgm_get_int_parameter(p, CFG_DB_NO_TABLES, &noTables));
   ndbrequire(!ndb_mgm_get_int_parameter(p, CFG_DICT_TABLE, &noTables));
-  ndbrequire(!ndb_mgm_get_int_parameter(p, CFG_DB_NO_ATTRIBUTES, &noAttribs));
   ndbrequire(!ndb_mgm_get_int_parameter(p, CFG_DIH_FRAG_CONNECT, &noFrags));
+  ndbrequire(!ndb_mgm_get_int_parameter(p,
+                                        CFG_LQH_FRAG,
+                                        &noDeleteLcpFile));
 
-  noAttribs++; //RT 527 bug fix
+  ndbrequire(noBackups == 1); /* To make sure we fix things if we allow other values */
 
+  /**
+   * On top of Backup records we need for LCP:
+   * 1 Backup record
+   * 5 files
+   *  2 CTL files for prepare and execute
+   *  2 Data files for prepare and execute
+   *  1 Data file for delete file process
+   * 2 tables
+   * 2 fragments
+   */
   c_nodePool.setSize(MAX_NDB_NODES);
   c_backupPool.setSize(noBackups + 1);
-  c_backupFilePool.setSize(3 * noBackups + 1);
-  c_tablePool.setSize(noBackups * noTables + 1);
+  c_backupFilePool.setSize(3 * noBackups +
+                           4 + (2*BackupFormat::NDB_MAX_FILES_PER_LCP));
+  c_tablePool.setSize(noBackups * noTables + 2);
   c_triggerPool.setSize(noBackups * 3 * noTables);
-  c_fragmentPool.setSize(noBackups * noFrags + 1);
+  c_fragmentPool.setSize(noBackups * noFrags + 2);
+  c_deleteLcpFilePool.setSize(noDeleteLcpFile);
 
-  c_tableMapSize = noBackups * noTables +1;
   c_tableMap = (Uint32*)allocRecord("c_tableMap",
                                     sizeof(Uint32),
-                                    c_tableMapSize);
+                                    noBackups * noTables);
 
-  for (Uint32 i = 0; i < c_tableMapSize; i++)
+  for (Uint32 i = 0; i < (noBackups * noTables); i++)
   {
     c_tableMap[i] = RNIL;
   }
 
   jam();
 
-  const Uint32 DEFAULT_WRITE_SIZE = (256 * 1024);
-  const Uint32 DEFAULT_MAX_WRITE_SIZE = (1024 * 1024);
-  const Uint32 DEFAULT_BUFFER_SIZE = (16 * 1024 * 1024);
+  Uint32 szWrite = BACKUP_DEFAULT_WRITE_SIZE;
+  Uint32 szDataBuf = BACKUP_DEFAULT_BUFFER_SIZE;
+  Uint32 maxWriteSize = szDataBuf;
 
-  Uint32 szDataBuf = DEFAULT_BUFFER_SIZE;
-  Uint32 szLogBuf = DEFAULT_BUFFER_SIZE;
-  Uint32 szWrite = DEFAULT_WRITE_SIZE;
-  Uint32 maxWriteSize = DEFAULT_MAX_WRITE_SIZE;
+  /**
+   * We set the backup data buffer size to 2M as hard coded. We add new code
+   * to ensure that we use as little as possible when performing LCP scans.
+   * This means that we continue the LCP scan until at least one file is
+   * ready to write. But if one file is ready to write and we have written
+   * more than 512 kB we will not continue the scan. We will however start
+   * a new LCP scan after a wait even if the buffer is full up to 512k.
+   * We ignore this check for LCP scans when the REDO log is at alert level.
+   * In this case we will continue writing until buffer is full.
+   *
+   * This behaviour ensures that we sustain optimal predictable latency as
+   * long as the REDO log is we are not at risk of running out of REDO log
+   * space. The higher buffer space is required to be able to keep up with
+   * loading massive amounts of data into NDB even with a very limited REDO
+   * log size.
+   *
+   * The minimum write size specifies the minimum size needed to collect
+   * in order to even consider writing the buffer to disk. It is also used
+   * when deciding to collect checkpoint data at priority A-level. If we
+   * reached the minimum write size we will only collect more checkpoint
+   * data at A-level if the REDO log is at any form of alert level.
+   *
+   * The maximum write size is the maximum size sent in one write to the
+   * file system. This is set to the same as the data buffer size. No need
+   * to make this configurable.
+   *
+   * We make the backup data buffer, write size and max write size hard coded.
+   * The sizes are large enough to provide enough bandwidth on hard drives.
+   * On SSD the defaults will be just fine. By limiting the backup data buffer
+   * size we avoid that we spend a lot of CPU resources to fill up the data
+   * buffer where there is anyways no room to write it out to the file.
+   *
+   * ndb_mgm_get_int_parameter(p, CFG_DB_BACKUP_DATA_BUFFER_MEM, &szDataBuf);
+   * ndb_mgm_get_int_parameter(p, CFG_DB_BACKUP_WRITE_SIZE, &szWrite);
+   * ndb_mgm_get_int_parameter(p, CFG_DB_BACKUP_MAX_WRITE_SIZE, &maxWriteSize);
+   */
 
-  ndb_mgm_get_int_parameter(p, CFG_DB_BACKUP_DATA_BUFFER_MEM, &szDataBuf);
+  Uint32 szLogBuf = BACKUP_DEFAULT_LOGBUFFER_SIZE;
   ndb_mgm_get_int_parameter(p, CFG_DB_BACKUP_LOG_BUFFER_MEM, &szLogBuf);
-  ndb_mgm_get_int_parameter(p, CFG_DB_BACKUP_WRITE_SIZE, &szWrite);
-  ndb_mgm_get_int_parameter(p, CFG_DB_BACKUP_MAX_WRITE_SIZE, &maxWriteSize);
-
   if (maxWriteSize < szWrite)
   {
     /**
@@ -300,24 +414,30 @@ Backup::execREAD_CONFIG_REQ(Signal* signal)
     g_eventLogger->info("BackupMemory parameter setting ignored,"
                         " BackupMemory deprecated");
   }
-  szMem = szDataBuf + szLogBuf;
 
   /**
    * We allocate szDataBuf + szLogBuf pages for Backups and
-   * szDataBuf pages for LCPs.
+   * szDataBuf * 16 pages for LCPs.
+   * We also need pages for 3 CTL files for LCP and one file for
+   * delete LCP process (2 per file),
+   * for backups the meta data file uses NO_OF_PAGES_META_FILE.
+   * We need to allocate an additional of 1 page because of a bug
+   * in ArrayPool.
    */
   Uint32 noPages =
-    (szMem + sizeof(Page32) - 1) / sizeof(Page32) +
-    (c_defaults.m_lcp_buffer_size + sizeof(Page32) - 1) / sizeof(Page32);
+    (szDataBuf + sizeof(Page32) - 1) / sizeof(Page32) +
+    (szLogBuf + sizeof(Page32) - 1) / sizeof(Page32) +
+    ((2 * BackupFormat::NDB_MAX_FILES_PER_LCP) * 
+      ((c_defaults.m_lcp_buffer_size + sizeof(Page32) - 1) /
+           sizeof(Page32)));
 
-  // We need to allocate an additional of 2 pages. 1 page because of a bug in
-  // ArrayPool and another one for DICTTABINFO.
-  c_pagePool.setSize(noPages + NO_OF_PAGES_META_FILE + 2, true); 
+  Uint32 seizeNumPages = noPages + (1*NO_OF_PAGES_META_FILE)+ 9;
+  c_pagePool.setSize(seizeNumPages, true);
 
   jam();
 
   { // Init all tables
-    SLList<Table> tables(c_tablePool);
+    Table_list tables(c_tablePool);
     TablePtr ptr;
     while (tables.seizeFirst(ptr)){
       new (ptr.p) Table(c_fragmentPool);
@@ -333,7 +453,7 @@ Backup::execREAD_CONFIG_REQ(Signal* signal)
   }
 
   {
-    SLList<BackupFile> ops(c_backupFilePool);
+    BackupFile_list ops(c_backupFilePool);
     BackupFilePtr ptr;
     while (ops.seizeFirst(ptr)){
       new (ptr.p) BackupFile(* this, c_pagePool);
@@ -347,7 +467,7 @@ Backup::execREAD_CONFIG_REQ(Signal* signal)
   }
   
   {
-    SLList<BackupRecord> recs(c_backupPool);
+    BackupRecord_sllist recs(c_backupPool);
     BackupRecordPtr ptr;
     while (recs.seizeFirst(ptr)){
       new (ptr.p) BackupRecord(* this, c_tablePool, 
