@@ -64,7 +64,7 @@ void Dbdih::initData()
     }
     while (c_masterActiveTakeOverList.first(ptr))
     {
-      releaseTakeOver(ptr, true);
+      releaseTakeOver(ptr, true, true);
     }
   }
   
@@ -85,22 +85,22 @@ void Dbdih::initData()
   c_sr_wait_to = false;
   c_2pass_inr = false;
   c_handled_master_take_over_copy_gci = 0;
-  
-  c_lcpTabDefWritesControl.init(MAX_CONCURRENT_LCP_TAB_DEF_FLUSHES);
+  c_start_node_lcp_req_outstanding = false;
 
+  c_lcpTabDefWritesControl.init(MAX_CONCURRENT_LCP_TAB_DEF_FLUSHES);
+  for (Uint32 i = 0; i < MAX_NDB_NODES; i++)
+  {
+    m_node_redo_alert_state[i] = RedoStateRep::NO_REDO_ALERT;
+  }
+  m_global_redo_alert_state = RedoStateRep::NO_REDO_ALERT;
+  m_master_lcp_req_lcp_already_completed = false;
+  
   c_shutdownReqNodes.clear();
 }//Dbdih::initData()
 
 void Dbdih::initRecords()
 {
   // Records with dynamic sizes
-  for (Uint32 i = 0; i < c_diverify_queue_cnt; i++)
-  {
-    c_diverify_queue[i].apiConnectRecord = (ApiConnectRecord*)
-      allocRecord("ApiConnectRecord",
-                  sizeof(ApiConnectRecord),
-                  capiConnectFileSize);
-  }
 
   connectRecord = (ConnectRecord*)allocRecord("ConnectRecord",
                                               sizeof(ConnectRecord), 
@@ -251,6 +251,7 @@ Dbdih::Dbdih(Block_context& ctx):
   addRecSignal(GSN_LCP_FRAG_REP, &Dbdih::execLCP_FRAG_REP);
   addRecSignal(GSN_START_LCP_REQ, &Dbdih::execSTART_LCP_REQ);
   addRecSignal(GSN_START_LCP_CONF, &Dbdih::execSTART_LCP_CONF);
+  addRecSignal(GSN_START_NODE_LCP_CONF, &Dbdih::execSTART_NODE_LCP_CONF);
   
   addRecSignal(GSN_READ_CONFIG_REQ, &Dbdih::execREAD_CONFIG_REQ, true);
   addRecSignal(GSN_UNBLO_DICTCONF, &Dbdih::execUNBLO_DICTCONF);
@@ -271,6 +272,8 @@ Dbdih::Dbdih(Block_context& ctx):
   addRecSignal(GSN_DICTSTARTCONF, &Dbdih::execDICTSTARTCONF);
   addRecSignal(GSN_NDB_STARTREQ, &Dbdih::execNDB_STARTREQ);
   addRecSignal(GSN_GETGCIREQ, &Dbdih::execGETGCIREQ);
+  addRecSignal(GSN_GET_LATEST_GCI_REQ, &Dbdih::execGET_LATEST_GCI_REQ);
+  addRecSignal(GSN_SET_LATEST_LCP_ID, &Dbdih::execSET_LATEST_LCP_ID);
   addRecSignal(GSN_DIH_RESTARTREQ, &Dbdih::execDIH_RESTARTREQ);
   addRecSignal(GSN_START_RECCONF, &Dbdih::execSTART_RECCONF);
   addRecSignal(GSN_START_FRAGCONF, &Dbdih::execSTART_FRAGCONF);
@@ -325,6 +328,8 @@ Dbdih::Dbdih(Block_context& ctx):
   addRecSignal(GSN_WAIT_GCP_REF, &Dbdih::execWAIT_GCP_REF);
   addRecSignal(GSN_WAIT_GCP_CONF, &Dbdih::execWAIT_GCP_CONF);
 
+  addRecSignal(GSN_REDO_STATE_REP, &Dbdih::execREDO_STATE_REP);
+
   addRecSignal(GSN_PREP_DROP_TAB_REQ, &Dbdih::execPREP_DROP_TAB_REQ);
   addRecSignal(GSN_DROP_TAB_REQ, &Dbdih::execDROP_TAB_REQ);
 
@@ -388,14 +393,6 @@ Dbdih::Dbdih(Block_context& ctx):
 
 Dbdih::~Dbdih()
 {
-  for (Uint32 i = 0; i<c_diverify_queue_cnt; i++)
-  {
-    deallocRecord((void **)&c_diverify_queue[i].apiConnectRecord,
-                  "ApiConnectRecord",
-                  sizeof(ApiConnectRecord),
-                  capiConnectFileSize);
-  }
-
   deallocRecord((void **)&connectRecord, "ConnectRecord",
                 sizeof(ConnectRecord), 
                 cconnectFileSize);
