@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2022, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -99,9 +99,7 @@ void Dbtux::execDBINFO_SCANREQ(Signal *signal)
 
     const size_t num_config_params =
       sizeof(pools[0].config_params) / sizeof(pools[0].config_params[0]);
-    const Uint32 numPools = NDB_ARRAY_SIZE(pools);
     Uint32 pool = cursor->data[0];
-    ndbrequire(pool < numPools);
     BlockNumber bn = blockToMain(number());
     while(pools[pool].poolname)
     {
@@ -226,7 +224,7 @@ Dbtux::printNode(TuxCtx & ctx,
   const Index& index = *c_indexPool.getPtr(frag.m_indexId);
   TreeHead& tree = frag.m_tree;
   NodeHandle node(frag);
-  selectNode(node, loc);
+  selectNode(ctx, node, loc);
   out << par.m_path << " " << node << endl;
   // check children
   PrintPar cpar[2];
@@ -440,8 +438,8 @@ operator<<(NdbOut& out, const Dbtux::ScanOp& scan)
   if (globalData.isNdbMtLqh)//TODO
     return out;
   {
-    DLFifoList<Dbtux::ScanLock>::Head head = scan.m_accLockOps;
-    LocalDLFifoList<Dbtux::ScanLock> list(tux->c_scanLockPool, head);
+    const Dbtux::ScanLock_fifo::Head& head = scan.m_accLockOps;
+    Dbtux::ConstLocal_ScanLock_fifo list(tux->c_scanLockPool, head);
     Dbtux::ScanLockPtr lockPtr;
     list.first(lockPtr);
     while (lockPtr.i != RNIL) {
@@ -460,7 +458,7 @@ operator<<(NdbOut& out, const Dbtux::ScanOp& scan)
     const Dbtux::Index& index = *tux->c_indexPool.getPtr(scan.m_indexId);
     Dbtux::KeyDataC keyBoundData(index.m_keySpec, true);
     Dbtux::KeyBoundC keyBound(keyBoundData);
-    tux->unpackBound(tux->c_ctx, scanBound, keyBound);
+    tux->unpackBound(tux->c_ctx.c_searchKey, scanBound, keyBound);
     out << " [scanBound " << dec << i;
     out << " " << keyBound.print(tux->c_ctx.c_debugBuffer, Dbtux::DebugBufferBytes);
     out << "]";
