@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -45,6 +45,38 @@ void Ndbcntr::initData()
   // schema trans
   c_schemaTransId = 0;
   c_schemaTransKey = 0;
+  init_local_sysfile();
+
+  m_any_lcp_started = false;
+  m_distributed_lcp_id = 0;
+  m_outstanding_wait_lcp = 0;
+  m_outstanding_wait_cut_redo_log_tail = 0;
+  m_set_local_lcp_id_reqs = 0;
+  m_received_wait_all = false;
+  m_wait_cut_undo_log_tail = false;
+  m_local_lcp_started = false;
+  m_local_lcp_completed = false;
+  m_full_local_lcp_started = false;
+  m_first_distributed_lcp_started = false;
+  m_distributed_lcp_started = false;
+  m_copy_fragment_in_progress = false;
+  m_max_gci_in_lcp = 0;
+  m_max_keep_gci = 0;
+  m_ready_to_cut_log_tail = false;
+  /**
+   * During initial start of Cluster we are executing an LCP before
+   * we have started the GCP protocol. The first GCI is 2, so to
+   * ensure that first LCP can complete we set m_max_completed_gci
+   * to 2 from the start although it isn't really completed yet.
+   */
+  m_max_completed_gci = 2;
+  m_initial_local_lcp_started = false;
+  m_lcp_id = 0;
+  m_local_lcp_id = 0;
+  m_global_redo_alert_state = RedoStateRep::NO_REDO_ALERT;
+  m_node_redo_alert_state = RedoStateRep::NO_REDO_ALERT;
+  for (Uint32 i = 0; i < MAX_NDBMT_LQH_THREADS; i++)
+    m_redo_alert_state[i] = RedoStateRep::NO_REDO_ALERT;
 }//Ndbcntr::initData()
 
 void Ndbcntr::initRecords() 
@@ -123,6 +155,8 @@ Ndbcntr::Ndbcntr(Block_context& ctx):
   addRecSignal(GSN_CHANGE_NODE_STATE_CONF, 
 	       &Ndbcntr::execCHANGE_NODE_STATE_CONF);
 
+  addRecSignal(GSN_REDO_STATE_REP, &Ndbcntr::execREDO_STATE_REP);
+
   addRecSignal(GSN_ABORT_ALL_REF, &Ndbcntr::execABORT_ALL_REF);
   addRecSignal(GSN_ABORT_ALL_CONF, &Ndbcntr::execABORT_ALL_CONF);
 
@@ -135,8 +169,37 @@ Ndbcntr::Ndbcntr(Block_context& ctx):
   addRecSignal(GSN_START_COPYREF, &Ndbcntr::execSTART_COPYREF);
   addRecSignal(GSN_START_COPYCONF, &Ndbcntr::execSTART_COPYCONF);
 
+  addRecSignal(GSN_WAIT_ALL_COMPLETE_LCP_REQ,
+               &Ndbcntr::execWAIT_ALL_COMPLETE_LCP_REQ);
+  addRecSignal(GSN_WAIT_COMPLETE_LCP_CONF,
+               &Ndbcntr::execWAIT_COMPLETE_LCP_CONF);
+  addRecSignal(GSN_START_LOCAL_LCP_ORD, &Ndbcntr::execSTART_LOCAL_LCP_ORD);
+  addRecSignal(GSN_START_DISTRIBUTED_LCP_ORD, &Ndbcntr::execSTART_DISTRIBUTED_LCP_ORD);
+  addRecSignal(GSN_COPY_FRAG_IN_PROGRESS_REP,
+               &Ndbcntr::execCOPY_FRAG_IN_PROGRESS_REP);
+  addRecSignal(GSN_COPY_FRAG_NOT_IN_PROGRESS_REP,
+               &Ndbcntr::execCOPY_FRAG_NOT_IN_PROGRESS_REP);
+  addRecSignal(GSN_LCP_ALL_COMPLETE_REQ, &Ndbcntr::execLCP_ALL_COMPLETE_REQ);
+  addRecSignal(GSN_CUT_UNDO_LOG_TAIL_CONF, &Ndbcntr::execCUT_UNDO_LOG_TAIL_CONF);
+  addRecSignal(GSN_CUT_REDO_LOG_TAIL_CONF, &Ndbcntr::execCUT_REDO_LOG_TAIL_CONF);
+  addRecSignal(GSN_RESTORABLE_GCI_REP, &Ndbcntr::execRESTORABLE_GCI_REP);
+  addRecSignal(GSN_UNDO_LOG_LEVEL_REP, &Ndbcntr::execUNDO_LOG_LEVEL_REP);
+  addRecSignal(GSN_SET_LOCAL_LCP_ID_REQ, &Ndbcntr::execSET_LOCAL_LCP_ID_REQ);
+
   addRecSignal(GSN_CREATE_NODEGROUP_IMPL_REQ, &Ndbcntr::execCREATE_NODEGROUP_IMPL_REQ);
   addRecSignal(GSN_DROP_NODEGROUP_IMPL_REQ, &Ndbcntr::execDROP_NODEGROUP_IMPL_REQ);
+  addRecSignal(GSN_READ_LOCAL_SYSFILE_REQ, &Ndbcntr::execREAD_LOCAL_SYSFILE_REQ);
+  addRecSignal(GSN_READ_LOCAL_SYSFILE_CONF, &Ndbcntr::execREAD_LOCAL_SYSFILE_CONF);
+  addRecSignal(GSN_WRITE_LOCAL_SYSFILE_REQ, &Ndbcntr::execWRITE_LOCAL_SYSFILE_REQ);
+  addRecSignal(GSN_WRITE_LOCAL_SYSFILE_CONF, &Ndbcntr::execWRITE_LOCAL_SYSFILE_CONF);
+  addRecSignal(GSN_FSOPENREF, &Ndbcntr::execFSOPENREF, true);
+  addRecSignal(GSN_FSOPENCONF, &Ndbcntr::execFSOPENCONF);
+  addRecSignal(GSN_FSREADREF, &Ndbcntr::execFSREADREF, true);
+  addRecSignal(GSN_FSREADCONF, &Ndbcntr::execFSREADCONF);
+  addRecSignal(GSN_FSWRITEREF, &Ndbcntr::execFSWRITEREF, true);
+  addRecSignal(GSN_FSWRITECONF, &Ndbcntr::execFSWRITECONF);
+  addRecSignal(GSN_FSCLOSEREF, &Ndbcntr::execFSCLOSEREF, true);
+  addRecSignal(GSN_FSCLOSECONF, &Ndbcntr::execFSCLOSECONF);
   
   initData();
   ctypeOfStart = NodeState::ST_ILLEGAL_TYPE;
