@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2015, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -52,6 +52,14 @@ public:
   void clear_buffer(Uint32 &rg, Ptr<GlobalPage> & ptr, Uint32 & cnt);
 
   AsyncIoThread* getThread() const { return m_thread;}
+  bool thread_bound() const
+  {
+    return m_thread_bound;
+  }
+  void set_thread_bound(bool value)
+  {
+    m_thread_bound = value;
+  }
 
   virtual Uint32 get_fileinfo() const { return 0; }
 private:
@@ -102,6 +110,8 @@ private:
   void detach(AsyncIoThread* thr);
 
   AsyncIoThread* m_thread; // For bound files
+  // Whether this file is one that will be/is bound to a thread
+  bool m_thread_bound;
 
 protected:
   size_t m_write_wo_sync;  // Writes wo/ sync
@@ -121,6 +131,10 @@ protected:
 
 public:
   SimulatedBlock& m_fs;
+#ifdef ERROR_INSERT
+  Uint32 m_error_insert;
+  void error_insert(Uint32 code) { m_error_insert = code; }
+#endif
 };
 
 inline
