@@ -167,7 +167,7 @@ void Thrman::execREAD_CONFIG_REQ(Signal *signal)
             new (sendThreadMeasurementPtr.p) SendThreadMeasurement();
         {
           jam();
-          LocalSendThreadMeasurement_list list_50ms(
+          Local_SendThreadMeasurement_fifo list_50ms(
             c_sendThreadMeasurementPool,
             sendThreadPtr.p->m_send_thread_50ms_measurements);
           list_50ms.addFirst(sendThreadMeasurementPtr);
@@ -178,7 +178,7 @@ void Thrman::execREAD_CONFIG_REQ(Signal *signal)
             new (sendThreadMeasurementPtr.p) SendThreadMeasurement();
         {
           jam();
-          LocalSendThreadMeasurement_list list_1sec(
+          Local_SendThreadMeasurement_fifo list_1sec(
             c_sendThreadMeasurementPool,
             sendThreadPtr.p->m_send_thread_1sec_measurements);
           list_1sec.addFirst(sendThreadMeasurementPtr);
@@ -189,7 +189,7 @@ void Thrman::execREAD_CONFIG_REQ(Signal *signal)
             new (sendThreadMeasurementPtr.p) SendThreadMeasurement();
         {
           jam();
-          LocalSendThreadMeasurement_list list_20sec(
+          Local_SendThreadMeasurement_fifo list_20sec(
             c_sendThreadMeasurementPool,
             sendThreadPtr.p->m_send_thread_20sec_measurements);
           list_20sec.addFirst(sendThreadMeasurementPtr);
@@ -474,7 +474,6 @@ Thrman::execOVERLOAD_STATUS_REP(Signal *signal)
 {
   Uint32 thr_no = signal->theData[0];
   Uint32 overload_status = signal->theData[1];
-  ndbrequire(thr_no < NDB_ARRAY_SIZE(m_thread_overload_status));
   m_thread_overload_status[thr_no].overload_status = (OverloadStatus)overload_status;
 
   Uint32 node_overload_level = 0;
@@ -843,7 +842,7 @@ Thrman::measure_cpu_usage(Signal *signal)
       c_sendThreadRecordPool.getPtr(sendThreadPtr, send_instance);
       {
         jam();
-        LocalSendThreadMeasurement_list list_50ms(c_sendThreadMeasurementPool,
+        Local_SendThreadMeasurement_fifo list_50ms(c_sendThreadMeasurementPool,
                             sendThreadPtr.p->m_send_thread_50ms_measurements);
         list_50ms.first(sendThreadMeasurementPtr);
         calculate_send_measurement(sendThreadMeasurementPtr,
@@ -857,7 +856,7 @@ Thrman::measure_cpu_usage(Signal *signal)
       if (elapsed_1sec > Uint64(1000 * 1000))
       {
         jam();
-        LocalSendThreadMeasurement_list list_1sec(c_sendThreadMeasurementPool,
+        Local_SendThreadMeasurement_fifo list_1sec(c_sendThreadMeasurementPool,
                             sendThreadPtr.p->m_send_thread_1sec_measurements);
         list_1sec.first(sendThreadMeasurementPtr);
         calculate_send_measurement(sendThreadMeasurementPtr,
@@ -871,7 +870,7 @@ Thrman::measure_cpu_usage(Signal *signal)
       if (elapsed_20sec > Uint64(20 * 1000 * 1000))
       {
         jam();
-        LocalSendThreadMeasurement_list list_20sec(c_sendThreadMeasurementPool,
+        Local_SendThreadMeasurement_fifo list_20sec(c_sendThreadMeasurementPool,
                             sendThreadPtr.p->m_send_thread_20sec_measurements);
         list_20sec.first(sendThreadMeasurementPtr);
         calculate_send_measurement(sendThreadMeasurementPtr,
@@ -1470,7 +1469,7 @@ Thrman::calculate_send_thread_load_last_second(Uint32 send_instance,
 
   c_sendThreadRecordPool.getPtr(sendThreadPtr, send_instance);
 
-  LocalSendThreadMeasurement_list list_50ms(c_sendThreadMeasurementPool,
+  Local_SendThreadMeasurement_fifo list_50ms(c_sendThreadMeasurementPool,
                          sendThreadPtr.p->m_send_thread_50ms_measurements);
   list_50ms.first(sendThreadMeasurementPtr);
 
@@ -2204,7 +2203,6 @@ Thrman::execDBINFO_SCANREQ(Signal* signal)
     Uint32 arr[NO_OF_BLOCKS];
     Uint32 len = mt_get_blocklist(this, arr, NDB_ARRAY_SIZE(arr));
     Uint32 pos = cursor->data[0];
-    ndbrequire(pos < NDB_ARRAY_SIZE(arr));
     for (; ; )
     {
       Ndbinfo::Row row(signal, req);
@@ -2443,7 +2441,7 @@ Thrman::execDBINFO_SCANREQ(Signal* signal)
         if (req.tableId == Ndbinfo::CPUSTAT_50MS_TABLEID)
         {
           jam();
-          LocalSendThreadMeasurement_list list_50ms(
+          Local_SendThreadMeasurement_fifo list_50ms(
             c_sendThreadMeasurementPool,
             sendThreadPtr.p->m_send_thread_50ms_measurements);
           list_50ms.last(sendThreadMeasurementPtr);
@@ -2451,7 +2449,7 @@ Thrman::execDBINFO_SCANREQ(Signal* signal)
         else if (req.tableId == Ndbinfo::CPUSTAT_1SEC_TABLEID)
         {
           jam();
-          LocalSendThreadMeasurement_list list_1sec(
+          Local_SendThreadMeasurement_fifo list_1sec(
             c_sendThreadMeasurementPool,
             sendThreadPtr.p->m_send_thread_1sec_measurements);
           list_1sec.last(sendThreadMeasurementPtr);
@@ -2459,7 +2457,7 @@ Thrman::execDBINFO_SCANREQ(Signal* signal)
         else if (req.tableId == Ndbinfo::CPUSTAT_20SEC_TABLEID)
         {
           jam();
-          LocalSendThreadMeasurement_list list_20sec(
+          Local_SendThreadMeasurement_fifo list_20sec(
             c_sendThreadMeasurementPool,
             sendThreadPtr.p->m_send_thread_20sec_measurements);
           list_20sec.last(sendThreadMeasurementPtr);
@@ -2531,7 +2529,7 @@ Thrman::execDBINFO_SCANREQ(Signal* signal)
         ndbrequire(sendThreadMeasurementPtr.i != RNIL);
         if (req.tableId == Ndbinfo::CPUSTAT_50MS_TABLEID)
         {
-          LocalSendThreadMeasurement_list list_50ms(
+          Local_SendThreadMeasurement_fifo list_50ms(
             c_sendThreadMeasurementPool,
             sendThreadPtr.p->m_send_thread_50ms_measurements);
           list_50ms.prev(sendThreadMeasurementPtr);
@@ -2543,7 +2541,7 @@ Thrman::execDBINFO_SCANREQ(Signal* signal)
         }
         else if (req.tableId == Ndbinfo::CPUSTAT_1SEC_TABLEID)
         {
-          LocalSendThreadMeasurement_list list_1sec(
+          Local_SendThreadMeasurement_fifo list_1sec(
             c_sendThreadMeasurementPool,
             sendThreadPtr.p->m_send_thread_1sec_measurements);
           list_1sec.prev(sendThreadMeasurementPtr);
@@ -2555,7 +2553,7 @@ Thrman::execDBINFO_SCANREQ(Signal* signal)
         }
         else if (req.tableId == Ndbinfo::CPUSTAT_20SEC_TABLEID)
         {
-          LocalSendThreadMeasurement_list list_20sec(
+          Local_SendThreadMeasurement_fifo list_20sec(
             c_sendThreadMeasurementPool,
             sendThreadPtr.p->m_send_thread_20sec_measurements);
           list_20sec.prev(sendThreadMeasurementPtr);
