@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2022, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -53,7 +53,7 @@
 
 #define CONSTRAINT_VIOLATION 893
 #define TUPLE_NOT_FOUND 626
-#define FK_NO_PARENT_ROW_EXISTS 255
+#define FK_NO_PARENT_ROW_EXISTS 21033
 
 static
 bool
@@ -153,12 +153,24 @@ Trix::execREAD_CONFIG_REQ(Signal* signal)
     m_ctx.m_config.getOwnConfigIterator();
   ndbrequire(p != 0);
 
+  c_maxUIBuildBatchSize = 64;
+  ndb_mgm_get_int_parameter(p, CFG_DB_UI_BUILD_MAX_BATCHSIZE,
+                            &c_maxUIBuildBatchSize);
+
+  c_maxFKBuildBatchSize = 64;
+  ndb_mgm_get_int_parameter(p, CFG_DB_FK_BUILD_MAX_BATCHSIZE,
+                            &c_maxFKBuildBatchSize);
+
+  c_maxReorgBuildBatchSize = 64;
+  ndb_mgm_get_int_parameter(p, CFG_DB_REORG_BUILD_MAX_BATCHSIZE,
+                            &c_maxReorgBuildBatchSize);
+
   // Allocate pool sizes
   c_theAttrOrderBufferPool.setSize(100);
   c_theSubscriptionRecPool.setSize(100);
   c_statOpPool.setSize(5);
 
-  DLList<SubscriptionRecord> subscriptions(c_theSubscriptionRecPool);
+  SubscriptionRecord_list subscriptions(c_theSubscriptionRecPool);
   SubscriptionRecPtr subptr;
   while (subscriptions.seizeFirst(subptr) == true) {
     new (subptr.p) SubscriptionRecord(c_theAttrOrderBufferPool);
@@ -545,9 +557,7 @@ void Trix::execDBINFO_SCANREQ(Signal *signal)
 
     const size_t num_config_params =
       sizeof(pools[0].config_params) / sizeof(pools[0].config_params[0]);
-    const Uint32 numPools = NDB_ARRAY_SIZE(pools);
     Uint32 pool = cursor->data[0];
-    ndbrequire(pool < numPools);
     BlockNumber bn = blockToMain(number());
     while(pools[pool].poolname)
     {
@@ -596,9 +606,12 @@ void Trix:: execBUILD_INDX_IMPL_REQ(Signal* signal)
   SubscriptionRecord* subRec;
   SectionHandle handle(this, signal);
 
-  if (ERROR_INSERTED_CLEAR(18000))
+  if (ERROR_INSERTED(18000) ||
+      ERROR_INSERTED(18003))
   {
-    sendSignalWithDelay(reference(), GSN_BUILD_INDX_IMPL_REQ, signal, 1000,
+    const Uint32 delay = (ERROR_INSERTED(18003)? 20000 : 1000);
+    CLEAR_ERROR_INSERT_VALUE;
+    sendSignalWithDelay(reference(), GSN_BUILD_INDX_IMPL_REQ, signal, delay,
                         signal->getLength(), &handle);
     DBUG_VOID_RETURN;
   }
@@ -626,7 +639,7 @@ void Trix:: execBUILD_INDX_IMPL_REQ(Signal* signal)
   subRec->indexType = buildIndxReq->indexType;
   subRec->sourceTableId = buildIndxReq->tableId;
   subRec->targetTableId = buildIndxReq->indexId;
-  subRec->parallelism = buildIndxReq->parallelism;
+  subRec->parallelism = c_maxUIBuildBatchSize;
   subRec->expectedConf = 0;
   subRec->subscriptionCreated = false;
   subRec->pendingSubSyncContinueConf = false;
@@ -1059,6 +1072,7 @@ void Trix::startTableScan(Signal* signal, SubscriptionRecPtr subRecPtr)
   subSyncReq->requestInfo = 0;
   subSyncReq->fragCount = subRec->fragCount;
   subSyncReq->fragId = subRec->fragId;
+  subSyncReq->batchSize = subRec->parallelism;
 
   if (subRec->m_flags & SubscriptionRecord::RF_NO_DISK)
   {
@@ -1572,7 +1586,7 @@ Trix::execCOPY_DATA_IMPL_REQ(Signal* signal)
   subRec->indexType = RNIL;
   subRec->sourceTableId = req->srcTableId;
   subRec->targetTableId = req->dstTableId;
-  subRec->parallelism = 16;
+  subRec->parallelism = c_maxReorgBuildBatchSize;
   subRec->expectedConf = 0;
   subRec->subscriptionCreated = false;
   subRec->pendingSubSyncContinueConf = false;
@@ -1710,7 +1724,7 @@ Trix::execBUILD_FK_IMPL_REQ(Signal* signal)
   subRec->indexType = RNIL;
   subRec->sourceTableId = req->childTableId;
   subRec->targetTableId = req->parentTableId;
-  subRec->parallelism = 16;
+  subRec->parallelism = c_maxFKBuildBatchSize;
   subRec->expectedConf = 0;
   subRec->subscriptionCreated = false;
   subRec->pendingSubSyncContinueConf = false;
@@ -2647,7 +2661,7 @@ Trix::statCleanPrepare(Signal* signal, StatOp& stat)
   subRec->targetTableId = RNIL;
   subRec->noOfIndexColumns = ao_size;
   subRec->noOfKeyColumns = 0;
-  subRec->parallelism = 16;
+  subRec->parallelism = 16;  // remains hardcoded for now
   subRec->fragCount = 0;
   subRec->fragId = ZNIL;
   subRec->syncPtr = RNIL;
@@ -2692,6 +2706,7 @@ Trix::statCleanPrepare(Signal* signal, StatOp& stat)
   default:
     boundCount = 0; /* Silence compiler warning */
     ndbrequire(false);
+    return; /* Silence compiler warning */
     break;
   }
   clean.m_boundSize = 3 * boundCount;
@@ -2857,7 +2872,7 @@ Trix::statScanPrepare(Signal* signal, StatOp& stat)
   subRec->targetTableId = RNIL;
   subRec->noOfIndexColumns = ao_size;
   subRec->noOfKeyColumns = 0;
-  subRec->parallelism = 16;
+  subRec->parallelism = 16;   // remains hardcoded for now
   subRec->fragCount = 0; // XXX Suma currently checks all frags
   subRec->fragId = req->fragId;
   subRec->syncPtr = RNIL;
@@ -3466,5 +3481,3 @@ operator<<(NdbOut& out, const Trix::StatOp& stat)
 
 
 BLOCK_FUNCTIONS(Trix)
-
-template void append(DataBuffer<15>&,SegmentedSectionPtr,SectionSegmentPool&);
