@@ -1,4 +1,4 @@
-/* Copyright (c) 2011-2013, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2011, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -32,6 +32,8 @@
 #include <NdbEventOperation.hpp>
 #include <NdbSleep.h>
 #include "NdbIndexStatImpl.hpp"
+#include <EventLogger.hpp>
+extern EventLogger * g_eventLogger;
 
 #undef min
 #undef max
@@ -70,8 +72,6 @@ NdbIndexStatImpl::init()
   // buffers
   m_keySpecBuf = 0;
   m_valueSpecBuf = 0;
-  m_keyDataBuf = 0;
-  m_valueDataBuf = 0;
   // cache
   m_cacheBuild = 0;
   m_cacheQuery = 0;
@@ -89,7 +89,7 @@ NdbIndexStatImpl::~NdbIndexStatImpl()
     m_query_mutex = 0;
   }
 }
- 
+
 // sys tables meta
 
 NdbIndexStatImpl::Sys::Sys(NdbIndexStatImpl* impl, Ndb* ndb) :
@@ -444,6 +444,7 @@ NdbIndexStatImpl::create_systables(Ndb* ndb)
       return -1;
 
 #ifdef VM_TRACE
+#ifdef NDB_USE_GET_ENV
     // test of schema trans
     {
       const char* p = NdbEnv_GetEnv("NDB_INDEX_STAT_ABORT_SYS_CREATE", (char*)0, 0);
@@ -453,6 +454,7 @@ NdbIndexStatImpl::create_systables(Ndb* ndb)
         return -1;
       }
     }
+#endif
 #endif
 
     if (dic->createTable(tab) == -1)
@@ -526,6 +528,7 @@ NdbIndexStatImpl::drop_systables(Ndb* ndb)
   {
 
 #ifdef VM_TRACE
+#ifdef NDB_USE_GET_ENV
     // test of schema trans
     {
       const char* p = NdbEnv_GetEnv("NDB_INDEX_STAT_ABORT_SYS_DROP", (char*)0, 0);
@@ -535,6 +538,7 @@ NdbIndexStatImpl::drop_systables(Ndb* ndb)
         return -1;
       }
     }
+#endif
 #endif
 
     if (dic->dropTableGlobal(*sys.m_sampletable) == -1)
@@ -549,7 +553,7 @@ NdbIndexStatImpl::drop_systables(Ndb* ndb)
     setError(dic->getNdbError().code, __LINE__);
     return -1;
   }
-    
+
   return 0;
 }
 
@@ -578,7 +582,7 @@ int
 NdbIndexStatImpl::check_systables(Ndb* ndb)
 {
   Sys sys(this, ndb);
-  
+
   if (check_systables(sys) == -1)
     return -1;
 
@@ -764,14 +768,6 @@ NdbIndexStatImpl::set_index(const NdbDictionary::Index& index,
     }
   }
 
-  // data buffers (rounded to word)
-  m_keyDataBuf = new Uint8 [m_keyData.get_max_len4()];
-  m_valueDataBuf = new Uint8 [m_valueData.get_max_len4()];
-  if (m_keyDataBuf == 0 || m_valueDataBuf == 0)
-  {
-    setError(NoMemError, __LINE__);
-    return -1;
-  }
   m_keyData.set_buf(m_keyDataBuf, m_keyData.get_max_len());
   m_valueData.set_buf(m_valueDataBuf, m_valueData.get_max_len());
 
@@ -787,8 +783,6 @@ NdbIndexStatImpl::reset_index()
   m_valueSpec.reset();
   delete [] m_keySpecBuf;
   delete [] m_valueSpecBuf;
-  delete [] m_keyDataBuf;
-  delete [] m_valueDataBuf;
   init();
 }
 
@@ -1179,6 +1173,28 @@ NdbIndexStatImpl::read_next(Con& con)
     return ret;
   }
 
+  /* varbinary column length is in first two bytes */
+  const Uint32 keyDataLen = m_keyDataBuf[0] + (m_keyDataBuf[1] << 8) + 2;
+  const Uint32 valueDataLen = m_valueDataBuf[0] + (m_valueDataBuf[1] << 8) + 2;
+
+  /* if the keyDataLen is more than that of buffer's max length or if the
+   * valueDataLen doesn't match the expected buffer length, report error */
+  if (keyDataLen > m_keyData.get_max_len() ||
+      valueDataLen != m_valueData.get_max_len())
+  {
+    g_eventLogger->error("ndb_index_stat key/value "
+                         "has different length than expected");
+    g_eventLogger->error("  index id : `%u`, table id : `%u`",
+                         m_indexId, m_tableId);
+    g_eventLogger->error("  key data length : `%u`, "
+                         "expected maximum length : `%u`",
+                         keyDataLen, m_keyData.get_max_len());
+    g_eventLogger->error("  value data length : `%u`, expected length : `%u`",
+                         valueDataLen, m_valueData.get_max_len());
+    setError(AlienUpdate, __LINE__, NdbPack::Error::InternalError);
+    return -1;
+  }
+
   /*
    * Key and value are raw data and little-endian.  Create the complete
    * NdbPack::Data instance and convert it to native-endian.
@@ -1273,10 +1289,13 @@ NdbIndexStatImpl::Cache::get_keyaddr(uint pos) const
   switch (m_addrLen) {
   case 4:
     addr += src[3] << 24;
+    // Fall through
   case 3:
     addr += src[2] << 16;
+    // Fall through
   case 2:
     addr += src[1] << 8;
+    // Fall through
   case 1:
     addr += src[0] << 0;
     break;
@@ -1296,10 +1315,13 @@ NdbIndexStatImpl::Cache::set_keyaddr(uint pos, uint addr)
   switch (m_addrLen) {
   case 4:
     dst[3] = (addr >> 24) & 0xFF;
+    // Fall through
   case 3:
     dst[2] = (addr >> 16) & 0xFF;
+    // Fall through
   case 2:
     dst[1] = (addr >> 8) & 0xFF;
+    // Fall through
   case 1:
     dst[0] = (addr >> 0) & 0xFF;
     break;
@@ -1746,6 +1768,7 @@ NdbIndexStatImpl::cache_hsort_sift(Cache& c, int i, int count)
   }
 }
 
+#ifdef ndb_index_stat_hsort_verify
 // verify heap property
 void
 NdbIndexStatImpl::cache_hsort_verify(Cache& c, int count)
@@ -1765,6 +1788,7 @@ NdbIndexStatImpl::cache_hsort_verify(Cache& c, int count)
     }
   }
 }
+#endif
 
 int
 NdbIndexStatImpl::cache_verify(const Cache& c)
@@ -2012,6 +2036,7 @@ NdbIndexStatImpl::convert_range(Range& range,
   }
 
 #ifdef VM_TRACE
+#ifdef NDB_USE_GET_ENV
   {
     const char* p = NdbEnv_GetEnv("NDB_INDEX_STAT_RANGE_ERROR", (char*)0, 0);
     if (p != 0 && strchr("1Y", p[0]) != 0)
@@ -2023,6 +2048,7 @@ NdbIndexStatImpl::convert_range(Range& range,
       }
     }
   }
+#endif
 #endif
 
   return 0;
@@ -2073,6 +2099,7 @@ NdbIndexStatImpl::query_stat(const Range& range, Stat& stat)
   NdbMutex_Unlock(m_query_mutex);
 
 #ifdef VM_TRACE
+#ifdef NDB_USE_GET_ENV
   {
     const char* p = NdbEnv_GetEnv("NDB_INDEX_STAT_SLOW_QUERY", (char*)0, 0);
     if (p != 0 && strchr("1Y", p[0]) != 0)
@@ -2081,6 +2108,7 @@ NdbIndexStatImpl::query_stat(const Range& range, Stat& stat)
       NdbSleep_MilliSleep(ms);
     }
   }
+#endif
 #endif
 
   // clients run these in parallel
@@ -2149,8 +2177,6 @@ NdbIndexStatImpl::query_interpolate(const Cache& c,
   const uint posH2 = stat2.m_pos;
   const uint cnt1 = bound1.m_data.get_cnt();
   const uint cnt2 = bound2.m_data.get_cnt();
-  const int side1 = bound1.m_bound.get_side();
-  const int side2 = bound2.m_bound.get_side();
   const uint mincnt = min(cnt1, cnt2);
   Uint32 numEq = 0; // of bound1,bound2
 
@@ -2206,7 +2232,8 @@ NdbIndexStatImpl::query_interpolate(const Cache& c,
         cnt2 == keyAttrs &&
         numEq == keyAttrs) {
       stat.m_rule[0] = "r2.1";
-      assert(side1 == -1 && side2 == +1);
+      assert(bound1.m_bound.get_side() == -1 &&
+             bound2.m_bound.get_side() == +1);
       assert(stat1.m_numEqL < keyAttrs && stat2.m_numEqH < keyAttrs);
       value.m_rir = c.get_rpk(posL1, posH1, keyAttrs - 1);
       for (uint k = 0; k < keyAttrs; k++)
@@ -2234,7 +2261,8 @@ NdbIndexStatImpl::query_interpolate(const Cache& c,
         cnt2 == keyAttrs &&
         numEq == keyAttrs) {
       stat.m_rule[0] = "r3.1";
-      assert(side1 == -1 && side2 == +1);
+      assert(bound1.m_bound.get_side() == -1 &&
+             bound2.m_bound.get_side() == +1);
       assert(stat1.m_numEqH == keyAttrs && stat2.m_numEqL == keyAttrs);
       value.m_rir = value2.m_rir - value1.m_rir;
       for (uint k = 0; k < keyAttrs; k++)
@@ -2572,18 +2600,15 @@ NdbIndexStatImpl::next_listener(Ndb* ndb)
 int
 NdbIndexStatImpl::drop_listener(Ndb* ndb)
 {
-  if (m_eventOp == 0)
-  {
-    setError(UsageError, __LINE__);
-    return -1;
-  }
-  if (ndb->dropEventOperation(m_eventOp) != 0)
+  if (m_eventOp != 0)
   {
     // NOTE! dropEventoperation always return 0
-    setError(ndb->getNdbError().code, __LINE__);
-    return -1;
+    int ret;
+    (void)ret; //USED
+    ret = ndb->dropEventOperation(m_eventOp);
+    assert(ret == 0);
+    m_eventOp = 0;
   }
-  m_eventOp = 0;
   return 0;
 }
 
@@ -2622,10 +2647,12 @@ NdbIndexStatImpl::setError(int code, int line, int extra)
   m_error.line = line;
   m_error.extra = extra;
 #ifdef VM_TRACE
+#ifdef NDB_USE_GET_ENV
   const char* p = NdbEnv_GetEnv("NDB_INDEX_STAT_ABORT_ON_ERROR", (char*)0, 0);
   if (p != 0 && strchr("1Y", p[0]) != 0)
     abort();
 #endif
+#endif
 }
 
 void
