@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2015, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -39,6 +39,8 @@ HugoTransactions::HugoTransactions(const NdbDictionary::Table& _tab,
 
   m_thr_count = 0;
   m_thr_no = -1;
+
+  m_empty_update = false;
 }
 
 HugoTransactions::~HugoTransactions(){
@@ -51,7 +53,8 @@ HugoTransactions::scanReadRecords(Ndb* pNdb,
 				  int abortPercent,
 				  int parallelism, 
 				  NdbOperation::LockMode lm,
-                                  int scan_flags)
+                                  int scan_flags,
+                                  int force_check_flag)
 {
   
   int                  retryAttempt = 0;
@@ -196,7 +199,8 @@ HugoTransactions::scanReadRecords(Ndb* pNdb,
     closeTransaction(pNdb);
 
     g_info << rows << " rows have been read" << endl;
-    if (records != 0 && rows != records){
+    if ((force_check_flag != 0 || records != 0) && rows != records)
+    {
       g_err << "Check expected number of records failed" << endl 
 	    << "  expected=" << records <<", " << endl
 	    << "  read=" << rows << endl;
@@ -512,7 +516,7 @@ restart:
 	  closeTransaction(pNdb);
 	  return NDBT_FAILED;
 	}
-	const int updates = calc.getUpdatesValue(&row) + 1;
+	const int updates = calc.getUpdatesValue(&row) + (m_empty_update? 0 : 1);
 	const int r = calc.getIdValue(&row);
         
   	for(a = 0; a<tab.getNoOfColumns(); a++){
@@ -545,8 +549,17 @@ restart:
       if( check == -1 ) {
 	closeTransaction(pNdb);
 	NDB_ERR(err);
-	if (err.status == NdbError::TemporaryError){
-	  NdbSleep_MilliSleep(50);
+        if (err.code == 499 || err.code == 631 ||   // Scan lock take over errors
+            err.status == NdbError::TemporaryError) // Other temporary errors
+        {
+          if (err.code == 410 || err.code == 1501)
+          {
+	    NdbSleep_MilliSleep(2000);
+          }
+          else
+          {
+	    NdbSleep_MilliSleep(300);
+          }
 	  goto restart;
 	}
 	setNdbError(err);
@@ -558,7 +571,9 @@ restart:
     if( check == -1 ) {
       closeTransaction(pNdb);
       NDB_ERR(err);
-      if (err.status == NdbError::TemporaryError){
+      if (err.code == 499 || err.code == 631 ||   // Scan lock take over errors
+          err.status == NdbError::TemporaryError) // Other temporary errors
+      {
 	NdbSleep_MilliSleep(50);
 	goto restart;
       }
@@ -630,11 +645,12 @@ HugoTransactions::loadTable(Ndb* pNdb,
                             bool oneTrans,
 			    int value,
 			    bool abort,
-                            bool abort_on_first_error)
+                            bool abort_on_first_error,
+                            int row_step)
 {
   return loadTableStartFrom(pNdb, 0, records, batch, allowConstraintViolation,
                             doSleep, oneTrans, value, abort,
-                            abort_on_first_error);
+                            abort_on_first_error, row_step);
 }
 
 int
@@ -647,7 +663,9 @@ HugoTransactions::loadTableStartFrom(Ndb* pNdb,
                                      bool oneTrans,
                                      int value,
                                      bool abort,
-                                     bool abort_on_first_error){
+                                     bool abort_on_first_error,
+                                     int row_step)
+{
   int             check;
   int             retryAttempt = 0;
   int             retryMax = 5;
@@ -668,23 +686,28 @@ HugoTransactions::loadTableStartFrom(Ndb* pNdb,
   
   //Uint32 orgbatch = batch;
   g_info << "|- Inserting records..." << endl;
-  for (int c=0 ; c<records; ){
+  for (int c=0 ; c < records; )
+  {
+    int key;
     bool closeTrans = true;
 
     if(c + batch > records)
       batch = records - c;
-    
+    key = c * row_step;
     if (retryAttempt >= retryMax){
       g_info << "Record " << c << " could not be inserted, has retried "
 	     << retryAttempt << " times " << endl;
       // Reset retry counters and continue with next record
       retryAttempt = 0;
-      c++;
+      if (row_step == 1)
+      {
+        c+= row_step;
+      }
     }
     if (doSleep > 0)
       NdbSleep_MilliSleep(doSleep);
 
-    //    if (first_batch || !oneTrans) {
+    //    if (first_batch || !oneTrans)
     if (first_batch || !pTrans) {
       first_batch = false;
       pTrans = pNdb->startTransaction();
@@ -702,8 +725,7 @@ HugoTransactions::loadTableStartFrom(Ndb* pNdb,
         return NDBT_FAILED;
       }
     }
-
-    if(pkInsertRecord(pNdb, c + startFrom, batch, value) != NDBT_OK)
+    if(pkInsertRecord(pNdb, key + startFrom, batch, value, row_step) != NDBT_OK)
     { 
       NDB_ERR(pTrans->getNdbError());
       setNdbError(pTrans->getNdbError());
@@ -749,12 +771,15 @@ HugoTransactions::loadTableStartFrom(Ndb* pNdb,
 	break;
 	
       case NdbError::TemporaryError:      
-        if (abort_on_first_error)
+        if (abort_on_first_error || row_step != 1)
         {
           return err.code;
         }
 	NDB_ERR(err);
-	NdbSleep_MilliSleep(50);
+        if (err.code == 410 || err.code == 1501)
+	  NdbSleep_MilliSleep(2000);
+        else
+	  NdbSleep_MilliSleep(50);
 	retryAttempt++;
         batch = 1;
 	continue;
@@ -793,7 +818,7 @@ HugoTransactions::loadTableStartFrom(Ndb* pNdb,
     }
     
     // Step to next record
-    c = c+batch; 
+    c = c+batch;
     retryAttempt = 0;
   }
 
@@ -1256,7 +1281,7 @@ HugoTransactions::pkUpdateRecords(Ndb* pNdb,
               return NDBT_FAILED;
             }
             
-            int updates = calc.getUpdatesValue(rows[0]) + 1;
+            int updates = calc.getUpdatesValue(rows[0]) + (m_empty_update? 0 : 1);
             
             /* Rows may not arrive in the order they were requested
              * (When multiple partitions scanned without ORDERBY)
@@ -1307,7 +1332,7 @@ HugoTransactions::pkUpdateRecords(Ndb* pNdb,
 	  return NDBT_FAILED;
 	}
 	
-	int updates = calc.getUpdatesValue(rows[b]) + 1;
+	int updates = calc.getUpdatesValue(rows[b]) + (m_empty_update? 0 : 1);
 	
 	if(pkUpdateRecord(pNdb, r+b, 1, updates) != NDBT_OK)
 	{
@@ -1325,7 +1350,10 @@ HugoTransactions::pkUpdateRecords(Ndb* pNdb,
       if (err.status == NdbError::TemporaryError){
 	NDB_ERR(err);
 	closeTransaction(pNdb);
-	NdbSleep_MilliSleep(50);
+        if (err.code == 410 || err.code == 1501)
+	  NdbSleep_MilliSleep(2000);
+        else
+	  NdbSleep_MilliSleep(300);
 	retryAttempt++;
 	continue;
       }
@@ -1444,7 +1472,7 @@ HugoTransactions::pkInterpretedUpdateRecords(Ndb* pNdb,
       return NDBT_FAILED;
     }
 
-    int updates = calc.getUpdatesValue(&row) + 1;
+    int updates = calc.getUpdatesValue(&row) + (m_empty_update? 0 : 1);
 
     NdbOperation* pUpdOp;
     pUpdOp = pTrans->getNdbOperation(tab.getName());	
@@ -1542,8 +1570,10 @@ HugoTransactions::pkDelRecords(Ndb* pNdb,
 			       int records,
 			       int batch,
 			       bool allowConstraintViolation,
-			       int doSleep){
-  // TODO Batch is not implemented
+			       int doSleep,
+                               int start_record,
+                               int step)
+{
   int deleted = 0;
   int                  r = 0;
   int                  retryAttempt = 0;
@@ -1583,6 +1613,7 @@ HugoTransactions::pkDelRecords(Ndb* pNdb,
       }
       NDB_ERR(err);
       setNdbError(err);
+      g_err << r << ": " << err.code << " " << err.message << endl;
       return NDBT_FAILED;
     }
 
@@ -1596,10 +1627,16 @@ HugoTransactions::pkDelRecords(Ndb* pNdb,
     if (timer_active)
       timer_start = NdbTick_getCurrentTicks();
 
-    if(pkDeleteRecord(pNdb, r, batch) != NDBT_OK)
+    int row = start_record + (r * step);
+    if(pkDeleteRecord(pNdb,
+                      row,
+                      batch,
+                      step) != NDBT_OK)
     {
-      NDB_ERR(pTrans->getNdbError());
-      setNdbError(pTrans->getNdbError());
+      const NdbError err = pTrans->getNdbError();
+      NDB_ERR(err);
+      setNdbError(err);
+      g_err << r << ": " << err.code << " " << err.message << endl;
       closeTransaction(pNdb);
       return NDBT_FAILED;
     }
@@ -1629,8 +1666,10 @@ HugoTransactions::pkDelRecords(Ndb* pNdb,
 	    break;
 	  }
 	}
+        g_err << "r = " << r << endl;
 	NDB_ERR(err);
 	setNdbError(err);
+	g_err << r << ": " << err.code << " " << err.message << endl;
 	closeTransaction(pNdb);
 	return NDBT_FAILED;
 	break;
@@ -1638,6 +1677,7 @@ HugoTransactions::pkDelRecords(Ndb* pNdb,
       default:
 	NDB_ERR(err);
 	setNdbError(err);
+	g_err << r << ": " << err.code << " " << err.message << endl;
 	closeTransaction(pNdb);
 	return NDBT_FAILED;
       }
@@ -2293,7 +2333,7 @@ HugoTransactions::indexUpdateRecords(Ndb* pNdb,
 	return NDBT_FAILED;
       }
       
-      int updates = calc.getUpdatesValue(rows[b]) + 1;
+      int updates = calc.getUpdatesValue(rows[b]) + (m_empty_update? 0 : 1);
       
       NdbOperation* pUpdOp;
       if(!ordered){
