@@ -391,7 +391,6 @@ print_diff(const Iter& iter)
   {
     if ((g_section == CFG_SECTION_CONNECTION &&
         (strcmp(ConfigInfo::m_ParamInfo[p]._section, "TCP") == 0 ||
-        strcmp(ConfigInfo::m_ParamInfo[p]._section, "SCI") == 0 ||
         strcmp(ConfigInfo::m_ParamInfo[p]._section, "SHM") == 0))
         ||
         (g_section == CFG_SECTION_NODE &&
@@ -528,7 +527,6 @@ helper(Vector<Apply*>& select, const char * str)
           ConfigInfo::m_ParamInfo[p]._fname);
       if ((g_section == CFG_SECTION_CONNECTION &&
         (strcmp(ConfigInfo::m_ParamInfo[p]._section, "TCP") == 0 ||
-          strcmp(ConfigInfo::m_ParamInfo[p]._section, "SCI") == 0 ||
           strcmp(ConfigInfo::m_ParamInfo[p]._section, "SHM") == 0))
         ||
         (g_section == CFG_SECTION_NODE &&
@@ -766,9 +764,6 @@ ConnectionTypeApply::apply(const Iter& iter)
     case CONNECTION_TYPE_TCP:
       printf("tcp");
       break;
-    case CONNECTION_TYPE_SCI:
-      printf("sci");
-      break;
     case CONNECTION_TYPE_SHM:
       printf("shm");
       break;
